### Script Generator for Ethernet terminal simulator ###
#
# Public domain software
#
# Coded by Yasusi Kanada
# 2014-6-15, 6-25, 2015-5-15, 5-18, 2016-6-18, 6-25

import sys, re, optparse

## Internal data structures ##

class NeighborDict(dict):	# hash table (dictionary) of neighbors
    def __missing__(self, key):
        self[key] = None
        return self[key]

class DeviceDict(dict):		# hash table (dictionary) of devices
    def __missing__(self, key):
        self[key] = { "type": None, "neighbors": NeighborDict(), "numPort": 0 }
        return self[key]

devices = DeviceDict()


## Options ##
parser = optparse.OptionParser()
parser.add_option("--mac", dest="isMac", action = "store_true")
parser.add_option("--path", dest="path", type = "string")
opts, args = parser.parse_args()


## Binary path ##

path = opts.path
if opts.isMac:
    pythonCommand = "python3"
    if path == None:
        path = "/Library/Frameworks/Python.framework/Versions/3.3/bin"
else:
    pythonCommand = "python"
    if path == None:
        path = "c:\python36;c:\python35;c:\python34;c:\python33;c:\python32;c:\python31"


## Compile network spec into internal form ##

def genInternal():
    pattern = re.compile(r"\s*(\w+)\s*(\w+)")

    myDeviceNum = 0
    for line in sys.stdin:    
        m = pattern.match(line)
        if m != None:
            command = m.group(1);
            argument = m.group(2);
            if command == "switch" or command == "repeater" or \
                    command == "terminal" or command == "server":
                myDeviceNum = int(argument)
                if myDeviceNum < 100 or 600 < myDeviceNum:
                    print("Error: Device number", myDeviceNum, "must be in 100..600")
                devices[myDeviceNum]["type"] = command
            elif command == "linkto":
                nextDeviceNum = int(argument)
                if nextDeviceNum < 100 or 999 < nextDeviceNum:
                    print("Error: Device number", nextDeviceNum, "must be in 100..999")
                elif nextDeviceNum == myDeviceNum:
                    print("Error: Duplicated device number", nextDeviceNum)
                myDevice = devices[myDeviceNum]
                nextDevice = devices[nextDeviceNum]
                myNeighbors = myDevice["neighbors"]
                nextNeighbors = nextDevice["neighbors"]
                if myNeighbors.get(nextDeviceNum) == None:	# not yet registered
                    myPort = myDevice["numPort"]
                    myDevice["numPort"] += 1
                    nextPort = nextDevice["numPort"]
                    nextDevice["numPort"] += 1
                    myNeighbors[nextDeviceNum] = \
                        { "myPort": myPort, "nextPort": nextPort }
                    nextNeighbors[myDeviceNum] = \
                        { "myPort": nextPort, "nextPort": myPort }
            elif command == "localMAC" or command == "remoteMAC":
                MACaddr = argument
                if len(MACaddr) != 12:
                    print("Error: MAC address length must be 12")
                devices[myDeviceNum][command] = MACaddr


## Generate scripts from internal form ##

def genScripts():
    for myDeviceNum in devices.keys():
        myDevice = devices[myDeviceNum]
        f = open("{}{}.bat".format(myDevice["type"], myDeviceNum),
                 "w", encoding="utf-8")

        if opts.isMac:
            print("PATH={}".format(path), file=f)
        else:
            print("path", path, file=f)

        neighbors = myDevice["neighbors"]
        if myDevice["type"] == "switch" or myDevice["type"] == "repeater":
            print(pythonCommand,
                  myDevice["type"] + ".py --monitor 0 --timeout 30 --dumpMAC --nports",
                  myDevice["numPort"], end="", file=f)
            for nextDeviceNum in myDevice["neighbors"].keys():
                next = neighbors[nextDeviceNum]
                myPort = next["myPort"]
                nextPort = next["nextPort"]
                print(" --lp{} {} --rp{} {}".\
                          format(myPort, 100 * myDeviceNum + myPort,
                                 myPort, 100 * nextDeviceNum + nextPort),
                      end="", file=f)
            print(file=f)
        elif myDevice["type"] == "terminal":
            if myDevice["localMAC"] == None:
                print("Error: Local MAC address not specified for terminal",
                      myDeviceNum)
            if myDevice["remoteMAC"] == None:
                print("Error: Remote MAC address not specified for terminal",
                      myDeviceNum)
            if myDevice["numPort"] > 1:
                print("Error: Two or more links specified for terminal", myDeviceNum)
            for nextDeviceNum in myDevice["neighbors"].keys():
                next = neighbors[nextDeviceNum]
                myPort = next["myPort"]
                nextPort = next["nextPort"]
                print(pythonCommand, "term.py --lm", myDevice["localMAC"],
                      "--rm", myDevice["remoteMAC"],
                      "--lp", 100 * myDeviceNum + myPort,
                      "--rp", 100 * nextDeviceNum + nextPort, file=f)
        elif myDevice["type"] == "server":
            if myDevice.get("localMAC") == None:
                print("Error: Local MAC address not specified for terminal",
                      myDeviceNum)
            if myDevice["numPort"] > 1:
                print("Error: Two or more links specified for terminal", myDeviceNum)
            for nextDeviceNum in myDevice["neighbors"].keys():
                next = neighbors[nextDeviceNum]
                myPort = next["myPort"]
                nextPort = next["nextPort"]
                print(pythonCommand, "server.py --lm", myDevice["localMAC"],
                      "--lp", 100 * myDeviceNum + myPort,
                      "--rp", 100 * nextDeviceNum + nextPort, file=f)
        else:
            print("Error: Unknown device type:", myDevice["type"])

        print("pause >nul", file=f)
        f.close()


genInternal()

genScripts()
